import json
import traceback
from logging.handlers import TimedRotatingFileHandler

import pymysql

from utils.BaseClass import *
from utils.DataBaseConfig import *
from utils.LogConfig import *

log_handler = TimedRotatingFileHandler("logs/ai_callback/ai_callback.log",
                                       when="midnight", backupCount=100)
log_handler.setFormatter(log_formatter)
logger = logging.getLogger('ai_callback_logger')
logger.addHandler(log_handler)
logger.setLevel(logging.DEBUG)
print('id of ai_callback_logger %s' % id(logger))
logger.info("ai_callback_server started!")


class AiCallBackAddCreative(tornado.web.RequestHandler):
    def post(self):
        res = {'code': 0,
               'message': "SUCCESS"}

        data = self.request.body
        data = str(data, 'utf8')
        data = json.loads(data, encoding='utf8')
        logger.info("***************************************  NEW REQUEST ***************************************")
        logger.info("raw data from request is %s" % data)

        try:
            db_con = pymysql.connect(host=host,
                                     port=port,
                                     user=user,
                                     password=passwd,
                                     database=db,
                                     charset=charset)
            cursor = db_con.cursor()
            for item in data['callbackData']:
                if item['code'] == 0:
                    creative_uuid = item['creative_uuid']
                    creative_id = item['creative_id']
                    unit_id = item['unit_id']
                    logger.info("on single update callback info: %s" % item)
                    cursor.execute("""UPDATE ctop_ai_kuaishou_creative_level_operation_record
                                    SET unit_id= %s, creative_id=%s
                                    WHERE creative_uuid = %s""", (unit_id, creative_id, creative_uuid))
                else:
                    logger.error("on single update callback info: %s" % item)

            db_con.commit()
            db_con.close()
        except Exception:
            res['code'] = -1
            res['message'] = 'FAIL'
            logger.error(traceback.format_exc())

        # 返回接口结果
        self.write(json.dumps(res))
        self.flush()


class AiCallBackAddGroup(tornado.web.RequestHandler):
    def post(self):
        res = {'code': 0,
               'message': "SUCCESS"}

        data = self.request.body
        data = str(data, 'utf8')
        data = json.loads(data, encoding='utf8')
        logger.info("***************************************  NEW REQUEST ***************************************")
        logger.info("raw data from request is %s" % data)

        try:
            db_con = pymysql.connect(host=host,
                                     port=port,
                                     user=user,
                                     password=passwd,
                                     database=db,
                                     charset=charset)
            cursor = db_con.cursor()
            for item in data['callbackData']:
                if item['code'] == 0:
                    group_uuid = item['group_uuid']
                    unit_id = item['unit_id']
                    group_create_time = item['group_create_time']
                    logger.info("on single update callback info: %s" % item)

                    cursor.execute("""UPDATE ctop_ai_kuaishou_unit_level_operation_record
                                    SET unit_id= %s, group_create_time=%s
                                    WHERE group_uuid = %s""", (unit_id, group_create_time, group_uuid))
                else:
                    logger.error("on single update callback info: %s" % item)

            db_con.commit()
            db_con.close()
        except Exception:
            res['code'] = -1
            res['message'] = 'FAIL'
            logger.error(traceback.format_exc())

        # 返回接口结果
        self.write(json.dumps(res))
        self.flush()